@@ -63,7 +63,8 @@ print_gsignal_struct (SCM gsignal, SCM port)
 
 SCM_DEFINE (scm_gtype_get_signals, "gtype-get-signals", 1, 0, 0,
 	    (SCM type),
-	    "")
+            "Returns a vector of signal names belonging to @var{type} and all "
+            "parent types.")
 #define FUNC_NAME s_scm_gtype_get_signals
 {
     guint *ids, n_ids, i;
@@ -362,6 +363,8 @@ scm_init_gnome_gobject_signals (void)
 	(scm_make_vtable_vtable (scm_makfrom0str ("pwpwpwpwpwpw"),
 				 SCM_INUM0, SCM_LIST1 (gsubr)));
 
+    scm_c_define ("<gsignal>", scm_gsignal_vtable);
+
     scm_c_define ("gsignal-id", SCM_MAKINUM (scm_si_gsignal_id));
     scm_c_define ("gsignal-name", SCM_MAKINUM (scm_si_gsignal_name));
     scm_c_define ("gsignal-interface_type", SCM_MAKINUM (scm_si_gsignal_interface_type));
