@@ -63,6 +63,9 @@
             wrap-boxed!
             wrap-pointer!
             wrap-opaque-pointer!
+            wrap-freeable-pointer!
+            wrap-refcounted-pointer!
+            wrap-structure!
             wrap-interface!
 
             ;; we extend the one from (g-wrap enumeration)
@@ -551,7 +554,7 @@ as in @code{wrap-enum!}."
   (let ((c-var (var value))
         (scm-var (scm-var value))
         (gtype-id (gtype-id type)))
-p    (list
+    (list
      "if (SCM_FALSEP (" scm-var "))\n"
      "  " c-var " = 0;\n"
      "else if (SCM_TYP16_PREDICATE (scm_tc16_gvalue, " scm-var ")\n"
@@ -575,19 +578,235 @@ p    (list
      "g_value_set_flags ((GValue *) SCM_SMOB_DATA (" scm-var "), " c-var ");\n")))
 
 
+(define-class <gobject-opaque-pointer> (<gw-wct> <gw-guile-rti-type>)
+  wct-var-name)
+
+(define-method (initialize (wct <gobject-opaque-pointer>) initargs)
+  (next-method)
+  (slot-set! wct 'wct-var-name
+	     (gen-c-tmp (string-append
+			 "wct_info_for"
+			 (any-str->c-sym-str (symbol->string (name wct)))))))
+
+(define-method (wrap-value-cg (wct <gobject-opaque-pointer>)
+			      (value <gw-value>)
+			      status-var
+			      (inlined? <boolean>))
+  (let ((wct-var (slot-ref wct 'wct-var-name))
+	(sv (scm-var value))
+	(cv (var value)))
+    (list
+     "if(" cv " == NULL) " sv " = SCM_BOOL_F;\n"
+     "else {\n"
+     sv " = gw_wcp_assimilate_ptr((void *) " cv ", " wct-var ");\n"
+     "}\n")))
+
+
+(define-method (unwrap-value-cg (wct <gobject-opaque-pointer>)
+				(value <gw-value>)
+				status-var
+				(inlined? <boolean>))
+  (let* ((wct-var (slot-ref wct 'wct-var-name))
+	 (sv (scm-var value))
+	 (c-var (var value))
+	 (unwrap-code
+	  (list "if (gw_wcp_is_of_type_p (" wct-var ", " sv "))\n"
+		"  " c-var " = gw_wcp_get_ptr (" sv ");\n"
+		"else\n"
+		`(gw:error ,status-var type ,(wrapped-var value)))))
+
+    (list
+     (if-typespec-option value 'null-ok
+			 (list "if (SCM_FALSEP (" sv "))\n"
+			       "  " c-var " = NULL;\n"
+			       "else " unwrap-code)
+			 unwrap-code))))
+
+(define-method (initializations-cg (wrapset <gobject-wrapset-base>)
+				   (wct <gobject-opaque-pointer>)
+				   error-var)
+  (let ((wct-var (slot-ref wct 'wct-var-name))
+	(wcp-type-name (symbol->string (name wct)))
+	(wcp-mark (wcp-mark-function wct))
+	(wcp-free (wcp-free-function wct))
+	(wcp-equal? (wcp-equal-predicate wct)))
+  (list
+   (next-method)
+
+   wct-var "= gw_wct_create (\"" wcp-type-name "\", " wcp-equal? ", NULL, "
+   wcp-mark ", " wcp-free ");\n"
+   "scm_c_define(\"" wcp-type-name "\", " wct-var ");\n")))
+
+(define (wct-var-decl-cg wct)
+  (list "static SCM "  (slot-ref wct 'wct-var-name) " = SCM_BOOL_F;\n"))
+
+(define-method (global-declarations-cg (wrapset <gobject-wrapset-base>)
+				       (wct <gobject-opaque-pointer>))
+  (wct-var-decl-cg wct))
+
+(define-method (client-global-declarations-cg (wrapset <gobject-wrapset-base>)
+					      (wct <gobject-opaque-pointer>))
+  (wct-var-decl-cg wct))
+
+(define-method (client-initializations-cg (wrapset <gobject-wrapset-base>)
+					  (wct <gobject-opaque-pointer>)
+					  error-var)
+  (let ((wct-var (slot-ref wct 'wct-var-name))
+	(wcp-type-name (symbol->string (name wct))))
+    (list
+     "    " wct-var " = scm_c_eval_string(\"" wcp-type-name "\");\n")))
+
 (define (wrap-opaque-pointer! ws ctype)
   "Define a wrapper for an opaque pointer with the C type @var{ctype}.
 It will not be possible to create these types from Scheme, but they can
 be received from a library, and passed as arguments to other calls into
 the library."
-  ;;(print-info "Opaque" ctype ctype ws) ; FIXME: Write to log file
-  (let ((type (wrap-as-wct!
-               ws
-               #:name (gtype-name->class-name ctype)
-               #:c-type-name ctype
-               #:c-const-type-name (string-append "const " ctype))))
-    (add-type-alias! ws ctype (name type))))
+  (let ((type (make <gobject-opaque-pointer>
+                #:name (gtype-name->class-name ctype)
+                #:c-type-name ctype
+                #:c-const-type-name (string-append "const " ctype))))
+    (add-type! ws type)
+    (add-type-alias! ws ctype (name type))
+    (print-info "Opaque" ctype ctype ws)
+    (add-module-export! ws (name type))
+    ;; don't export, unlike wrap-as-wct!
+    type))
 
+
+(define-class <gobject-freeable-pointer> (<gobject-opaque-pointer>)
+  (free-function #:init-keyword #:free-function))
+
+(define-method (global-declarations-cg (wrapset <gobject-wrapset-base>)
+				       (wct <gobject-freeable-pointer>))
+  (list "static size_t " (wcp-free-function wct) " (void *wcp) {\n"
+        (slot-ref wct 'free-function) " (wcp); return 0;\n"
+        "}"
+        (next-method)))
+
+(define (wrap-freeable-pointer! ws ctype free)
+  "foo"
+  (let* ((type (make <gobject-freeable-pointer>
+                 #:name (gtype-name->class-name ctype)
+                 #:c-type-name (string-append ctype "*")
+                 #:c-const-type-name (string-append "const " ctype "*")
+                 #:wcp-free-function (string-append "_wcp_free_" free)
+                 #:free-function free)))
+    (print-info "Freeable" ctype ctype ws)
+    (add-type! ws type)
+    (add-type-alias! ws (string-append ctype "*") (name type))
+    type))
+
+
+(define-class <gobject-ref-pointer> (<gobject-freeable-pointer>)
+  (ref-function #:init-keyword #:ref-function))
+
+(define-method (wrap-value-cg (wct <gobject-ref-pointer>)
+			      (value <gw-value>)
+			      status-var
+			      (inlined? <boolean>))
+  ;; potential FIXME, caller-owned unwraps
+  (let ((sv (scm-var value))
+        (cv (var value))
+        (ref (slot-ref wct 'ref-function)))
+    (append
+     (if-typespec-option value 'callee-owned
+                         (list "if (" cv ")\n  " ref " (" cv ");\n")
+                         '())
+     (next-method))))
+
+(define (wrap-refcounted-pointer! ws ctype ref unref)
+  "foo"
+  (let* ((type (make <gobject-ref-pointer>
+                 #:name (gtype-name->class-name ctype)
+                 #:c-type-name (string-append ctype "*")
+                 #:c-const-type-name (string-append "const " ctype "*")
+                 #:wcp-free-function (string-append "_wcp_free_" unref)
+                 #:ref-function ref
+                 #:free-function unref)))
+    (print-info "RefPtr" ctype ctype ws)
+    (add-type! ws type)
+    (add-type-alias! ws (c-type-name type) (name type))
+    type))
+
+
+(define-class-with-docs <gobject-structure-type> (<gw-type>)
+  "foo docs here."
+  (c-type-name #:init-keyword #:c-type-name #:getter c-type-name)
+  (wrap-func #:init-keyword #:wrap-func #:getter wrap-func)
+  (unwrap-func #:init-keyword #:unwrap-func #:getter unwrap-func))
+
+(define-method (check-typespec-options (type <gobject-structure-type>) (options <list>))
+  (let ((remainder options))
+    (define (del*! . syms)
+      (set! remainder (lset-difference eq? remainder syms)))
+    (del*! 'const 'out 'unspecialized)
+    (cond
+     ((and (memq 'caller-owned remainder) (memq 'callee-owned remainder))
+      (raise-bad-typespec type options "both caller and callee owned"))
+     ((not (or (memq 'caller-owned remainder) (memq 'callee-owned remainder)))
+      (raise-bad-typespec type options "must be caller or callee owned"))
+     (else
+      (apply del*! 'caller-owned 'callee-owned 
+             (slot-ref type 'allowed-options))
+      (if (not (null? remainder))
+          (raise-bad-typespec type options
+                              "spurious options in RTI type: ~S" remainder))))))
+
+(define-method (c-type-name (type <gobject-structure-type>)
+                            (typespec <gw-typespec>))
+  (c-type-name type))
+
+(define-method (make-typespec (type <gobject-structure-type>) (options <list>))
+  (next-method type (cons 'unspecialized options)))
+
+(define-method (unwrap-value-cg (type <gobject-structure-type>)
+                                (value <gw-value>)
+                                status-var)
+  (let ((c-var (var value))
+        (scm-var (scm-var value)))
+    (unwrap-null-checked
+     value status-var
+     (list
+      ;; memset c-var to 0 ?
+      (unwrap-func type) " (" scm-var ", &" c-var ");\n"))))
+
+(define-method (call-arg-cg (type <gobject-structure-type>)
+                            (value <gw-value>))
+  (list "&" (var value)))
+
+(define-method (wrap-value-cg (type <gobject-structure-type>)
+                              (value <gw-value>)
+                              status-var)
+  (let ((c-var (var value))
+        (scm-var (scm-var value)))
+    (list
+     scm-var " = " (wrap-func type) " (&" c-var ");\n")))
+
+(define (wrap-structure! ws ctype wrap unwrap)
+  "Define a wrapper for structure values of type @var{ctype}.
+
+@var{wrap} and @var{unwrap} are the names of C functions to convert a C
+structure to Scheme and vice versa, respectively. When in a function
+call, parameters of this type of the form `@var{StructName}*' are
+interpreted as `out' parameters, while `const-@var{StructName}*' are
+treated as `in' parameters.
+
+Note that @var{ctype} should be the type of the structure, not a pointer
+to the structure."
+  (let* ((type (make <gobject-structure-type>
+                 #:name (gtype-name->class-name ctype)
+                 #:c-type-name ctype
+                 #:wrap-func wrap
+                 #:unwrap-func unwrap)))
+    (print-info "Structure" ctype ctype ws)
+    (add-type! ws type)
+    (add-type-rule! ws (string-append ctype "*")
+                    (list (name type) 'caller-owned 'out))
+    (add-type-rule! ws (string-append "const-" ctype "*")
+                    (list (name type) 'callee-owned))
+    type))
+
+
 ;; Used for functions that operate on classes, e.g.
 ;; gtk_widget_class_install_style_property,
 ;; gst_element_class_get_pad_template
