@@ -1,4 +1,5 @@
 #! /bin/sh
+# -*- scheme -*-
 exec guile-gnome-0 -s $0 "$@"
 !#
 ;; guile-gnome
@@ -21,7 +22,8 @@ exec guile-gnome-0 -s $0 "$@"
 ;; 59 Temple Place - Suite 330        Fax:    +1-617-542-2652
 ;; Boston, MA  02111-1307,  USA       gnu@gnu.org
 
-(use-modules (gnome gtk)
+(use-modules (oop goops)
+             (gnome gtk)
              (gnome glade))
 
 (define (frobify button)
